<?php

/* session_2.php: Additional functions for settings, contacts, etc.
*
    Copyright (C) 2002  Hastymail Development group

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; Version 2
    See the COPYING file for complete information.

* $Id: session_2.php,v 1.7 2003/08/19 04:25:33 sailfrog Exp $
*/

/* read in the contacts list from ldap book defined in hastymail.conf
*/
function hm_get_ldap_contacts() {
    $contacts = array();
    $str = $_SESSION['str'];
    if ($ldapp= @ldap_connect($_SESSION['config']['ldap_server'], 
                             $_SESSION['config']['ldap_port'])) {
        if (@ldap_bind($ldapp)) {
            if (isset($_SESSION['config']['ldap_search_term']) &&
                $_SESSION['config']['ldap_search_term'] != '') {
                $search = $_SESSION['config']['ldap_search_term'];
            }
            else {
                $search = 'cn=*';
            }
            $ldap_attr = @ldap_search($ldapp, $_SESSION['config']['ldap_base_dn'], 
                         $search);
            $info = ldap_get_entries($ldapp, $ldap_attr);
            ldap_close($ldapp);
            array_shift($info);
            $i = 0;
            foreach ($info as $array) {
                if (isset($array['mail'][0]) && strstr($array['mail'][0], '@')) {
                    $contacts[$i] = array( $i, trim($array['mail'][0]), 
                                    trim($array['cn'][0], ''));
                    $i++;
                }
            }
        }
        else {
            ldap_close($ldapp);
            $_SESSION['error'] = $str[197];
        }
    }
    else {
        $_SESSION['error'] = $str[198];
    }

    return $contacts;
}

/* read in the contacts list
   $user = username
   $dir  = settings dir
   returns contacts array where each element is an array of one contact's
   attributes. This array has numeric keys
*/
function hm_get_contacts($user, $dir) {
    $contacts = array();
    if ($contacts_file = @fopen ($dir.strtolower($user).'.contacts', 'r')) {
        while (!feof ($contacts_file)) {
            $line = fgets($contacts_file, 1024);
            $parts = preg_split("/\|/", $line, 4);
            if (count($parts) == 4) {
                $contacts[$parts[0]] = $parts;
            }
        }
    }
    return $contacts;
}

/* sort the contact list 
   $contacts = the contacts array
   returns $new_contacts which is the same as contacts but sorted
   alphabeticaly
*/
function hm_sort_contacts($contacts) {
    $i = 0;
    $new_contacts = array();
    if (!function_exists('cmpsort')) {
        function cmpsort ($a, $b) {
            return strcmp(strtolower($a[2]), strtolower($b[2]));
        }
    }
    usort($contacts, "cmpsort");
    foreach ($contacts as $index => $array) {
        $new_contacts[$i] = $array;
        $new_contacts[$i][0] = $i;
        $i++;
    }
    return $new_contacts;
}

/* get the users settings 
   $user = username
   $dir  = settings directory
   returns $settings array where each key is a setting name and
   each value is the setting value, or array of values.
*/
function hm_get_settings($user, $dir) {
    $settings = array();
    if (!is_readable($dir.strtolower($user).'.settings')) {
        $_SESSION['first_login'] = true;
    }
    if ($settings_file = @fopen ($dir.strtolower($user).'.settings', 'r')) {
        while (!feof ($settings_file)) {
            $line = fgets($settings_file, 1024);
            if (!preg_match("/^#/", $line)) {
                $pair = preg_split("/=/", $line);
                if (isset($pair[0]) && isset($pair[1])) {
                    switch ($pair[0]) {
                        case 'hidden':
                            $settings['hidden'][] = trim($pair[1]);
                            break;
                        case 'sort':
                            $parts = preg_split("/\|/", $pair[1], 2);
                            if (isset($parts[0]) && $parts[1]) {
                                $settings['sort'][$parts[0]] = trim($parts[1]);
                            }
                            break;
                        default:
                            $settings[$pair[0]] = trim($pair[1]);
                            break;
                    }
                }
            }
        }
        fclose($settings_file);
    }
    for ($i=0;$i<4;$i++) {
        $_SESSION['profiles'][$i]['sig'] = '';
        $part = 'sig';
        if ($i != 0) {
            $part .= '_'.$i;
        }
        $_SESSION['profiles'][$i]['sig'] = @implode ('', @file($dir.strtolower($user).
                                           '.'.$part));
    }
    return $settings;
}

/* write the contacts to file
   $user     = username
   $dir      = settings directory
   $contacts = the contacts array
   returns $result, which is true on success false on failure
*/
function hm_write_contacts($user, $dir, $contacts) {
    $result = false;
    $contacts = hm_sort_contacts($contacts);
    if ($contacts_file = @fopen ($dir.strtolower($user).'.contacts', 'w')) {
        $i = 0;
        foreach ($contacts as $array) {
            $line = $i.'|'.trim($array[1]).'|'.trim($array[2]).'|'.trim($array[3]);
            str_replace("\n", '', $line);
            fputs($contacts_file, trim($line)."\n");
            $i++;
        }
        fclose($contacts_file);
        $result = true;
    }
    return $result;
}

/* write the users settings to file
   $user = username
   $dir  = settings dir
   does not return anything 
*/  
function hm_write_settings($user, $dir) {
    $settings = $_SESSION['settings'];
    if (isset($settings['sort'])) {
        $sorts = $settings['sort'];
        $settings['sort'] = array();
        foreach($sorts as $index => $value) {
            $settings['sort'][] = $index.'|'.$value;
        }
    }
    if ($settings_file = @fopen ($dir.strtolower($user).'.settings', 'w')) {
        foreach ($settings as $index => $value) {
            if (is_array($value)) {
                foreach ($value as $element) {
                    fputs($settings_file, $index."=".$element."\n");
                }
            }
            else { 
                fputs($settings_file, $index."=".$value."\n");
            }
        }
        fclose($settings_file);
    }
    for ($i=0;$i<4;$i++) {
        $part = 'sig';
        if ($i != 0) {
            $part .= '_'.$i;
        }
        if (isset($_SESSION['profiles'][$i]['sig'])) {
            $sig_file = @fopen ($dir.strtolower($user).'.'.$part, 'w');
            if ($sig_file) {
                fputs($sig_file, $_SESSION['profiles'][$i]['sig']);
                fclose($sig_file);
            }
        }
    }
}

?>
